#ifndef __DILLODICACHE_H__
#define __DILLODICACHE_H__

#include "bitvec.h"
#include "image.h"
#include "cache.h"

/* These will reflect the entry's "state" */
typedef enum {
   DIC_Empty,      /* Just created the entry */
   DIC_SetParms,   /* Parameters set */
   DIC_SetCmap,    /* Color map set */
   DIC_Write,      /* Feeding the entry */
   DIC_Close,      /* Whole image got! */
   DIC_Abort       /* Image transfer aborted */
} DicEntryState;

typedef struct _DICacheEntry DICacheEntry;

struct _DICacheEntry {
   gint Key;               /* A primary Key for this entry */
   DilloUrl *url;          /* Image URL for this entry */
   gint width, height;     /* As taken from image data */
   DilloImgType type;      /* Image type */
   guchar *cmap;           /* Color map */
   guchar *ImageBuffer;    /* Decompressed buffer */
   size_t TotalSize;       /* Amount of memory the image takes up */
   gint Y;                 /* Current decoding row */
   bitvec_t *BitVec;       /* Bit vector for decoded rows */
   DicEntryState State;    /* Current status for this entry */
};


void a_Dicache_init (void);

DICacheEntry *a_Dicache_get_entry(const DilloUrl *Url);
DICacheEntry *a_Dicache_get_entry_by_key(gint Key);
DICacheEntry *a_Dicache_add_entry(const DilloUrl *Url);

void a_Dicache_callback(gint Op, CacheClient_t *Client);

void a_Dicache_set_parms(gint DicEntryKey, DilloImage *Image,
                         gint width, gint height, DilloImgType type);
void a_Dicache_set_cmap(gint DicKey, DilloImage *Image,
                        const guchar *cmap, gint num_colors, gint bg_index);
void a_Dicache_write(DilloImage *Image, gint DicKey,
                     const guchar *buf, gint x, gint Y);
void a_Dicache_close(gint DicKey, CacheClient_t *Client);

void a_Dicache_freeall(void);
void a_Dicache_remove(const DilloUrl *Url, gint DicKey);

#endif /* __DILLODICACHE_H__ */
